# Detect the distribution in use
%global __despace head -n 1 | tr -d '[:space:]' | sed -e 's/[(].*[)]//g'
%global __lower4 cut -c 1-4 | tr '[:upper:]' '[:lower:]'
%global __distfile %([ -f /etc/SuSE-release ] && echo /etc/SuSE-release || echo /etc/redhat-release)
%global __distinit %(sed -e 's/ release .*//' -e 's/\\([A-Za-z]\\)[^ ]*/\\1/g' %{__distfile} | %{__despace} | %{__lower4})
%global __distvers %(sed -e 's/.* release \\([^. ]*\\).*/\\1/' %{__distfile} | %{__despace})
# Identify CentOS Linux and Scientific Linux as rhel
%if "%{__distinit}" == "c" || "%{__distinit}" == "cl" || "%{__distinit}" == "sl" || "%{__distinit}" == "sls"
%global __distinit rhel
%endif
# Dist tag for Fedora is still "fc"
%if "%{__distinit}" == "f"
%global __distinit fc
%endif

# We need to patch out the requirement for a recent List::Util on older
# distributions and use List::MoreUtils instead
%global old_list_util 0%{?fedora} < 21 && 0%{?rhel} < 8

# We need Test::More ≥ 0.96, so use a bundled one if we don't have it
# Cannot create sub-package perl-Test-Specio with Test::More < 0.96 either, since it requires at least that version
%global old_test_more %(perl -MTest::More -e 'print (($Test::More::VERSION < 0.96) ? 1 : 0);' 2>/dev/null || echo 0)

# Note: optional dependency Sub::Util is packaged with Scalar-List-Utils ≥ 1.40 (August 2014, F-21 onwards)

# Provides/Requires filtering is different from rpm 4.9 onwards
%global rpm49 %(rpm --version | perl -p -e 's/^.* (\\d+)\\.(\\d+).*/sprintf("%d.%03d",$1,$2) ge 4.009 ? 1 : 0/e' 2>/dev/null || echo 0)

Name:		perl-Specio
Version:	0.36
Release:	1.%{__distinit}%{__distvers}
Summary:	Type constraints and coercions for Perl
License:	Artistic 2.0
Group:		Development/Libraries
URL:		http://search.cpan.org/dist/Specio/
Source0:	http://search.cpan.org/CPAN/authors/id/D/DR/DROLSKY/Specio-%{version}.tar.gz
Source1:	T-M-0.96.tar.gz
Patch0:		Specio-0.36-old-List::Util.patch
BuildRoot:	%{_tmppath}/%{name}-%{version}-%{release}-root-%(id -nu)
BuildArch:	noarch
# Module Build
BuildRequires:	coreutils
BuildRequires:	findutils
BuildRequires:	make
BuildRequires:	perl
%if 0%{?fedora} > 20 || 0%{?rhel} > 7
BuildRequires:	perl-generators
%endif
BuildRequires:	perl(ExtUtils::MakeMaker)
# Module Runtime
BuildRequires:	perl(B)
BuildRequires:	perl(Carp)
BuildRequires:	perl(Devel::StackTrace)
BuildRequires:	perl(Eval::Closure)
BuildRequires:	perl(Exporter)
BuildRequires:	perl(IO::File)
%if %{old_list_util}
BuildRequires:	perl(List::Util)
BuildRequires:	perl(List::MoreUtils)
%else
BuildRequires:	perl(List::Util) >= 1.33
%endif
BuildRequires:	perl(Module::Runtime)
BuildRequires:	perl(MRO::Compat)
BuildRequires:	perl(overload)
BuildRequires:	perl(parent)
BuildRequires:	perl(re)
BuildRequires:	perl(Ref::Util) >= 0.112
BuildRequires:	perl(Role::Tiny) >= 1.003003
BuildRequires:	perl(Role::Tiny::With)
BuildRequires:	perl(Scalar::Util)
BuildRequires:	perl(Storable)
BuildRequires:	perl(strict)
BuildRequires:	perl(Test::Fatal)
BuildRequires:	perl(Test::More)
BuildRequires:	perl(version) >= 0.83
BuildRequires:	perl(warnings)
# Test Suite
BuildRequires:	perl(File::Spec)
BuildRequires:	perl(lib)
BuildRequires:	perl(open)
BuildRequires:	perl(Test::Needs)
BuildRequires:	perl(utf8)
# Optional Tests (note: Moose/Mouse use DateTime and hence Specio in their test suites)
BuildRequires:	perl(CPAN::Meta) >= 2.120900
BuildRequires:	perl(CPAN::Meta::Prereqs)
%if 0%{!?perl_bootstrap:1}
BuildRequires:	perl(Moo)
BuildRequires:	perl(Moose) >= 2.1207
BuildRequires:	perl(Mouse)
%endif
BuildRequires:	perl(namespace::autoclean)
# Dependencies
Requires:	perl(:MODULE_COMPAT_%(eval "`perl -V:version`"; echo $version))
Requires:	perl(Ref::Util) >= 0.112

# Optional Functionality
%if 0%{?fedora} > 20 || 0%{?rhel} > 7
BuildRequires:	perl(Sub::Util) >= 1.40
Requires:	perl(Sub::Util) >= 1.40
%else
BuildRequires:	perl(Sub::Name)
Requires:	perl(Sub::Name)
%endif

# Avoid provides for private packages
%if %{rpm49}
%global __provides_exclude ^perl\\(_T::.*\\)
%else
%global provfilt /bin/sh -c "%{__perl_provides} | grep -v '^perl[(]_T::.*[)]'"
%global __perl_provides %{provfilt}
%endif

%description
The Specio distribution provides classes for representing type constraints
and coercion, along with syntax sugar for declaring them.

Note that this is not a proper type system for Perl. Nothing in this
distribution will magically make the Perl interpreter start checking a value's
type on assignment to a variable. In fact, there's no built-in way to apply a
type to a variable at all.

Instead, you can explicitly check a value against a type, and optionally coerce
values to that type.

%if ! %{old_test_more}
%package -n perl-Test-Specio
Summary:	Test helpers for Specio
License:	Artistic 2.0
Group:		Development/Libraries
Requires:	%{name} = %{version}-%{release}

%description -n perl-Test-Specio
This package provides some helper functions and variables for testing Specio
types.
%endif

%prep
%setup -q -n Specio-%{version}

# We need to patch out the requirement for a recent List::Util on older
# distributions and use List::MoreUtils instead
%if %{old_list_util}
%patch0
%endif

# We need Test::More ≥ 0.96, so use a bundled one if we don't have it
%if %{old_test_more}
tar xfz %{SOURCE1}
find inc/Test -type f >> MANIFEST
%endif

%build
perl Makefile.PL INSTALLDIRS=vendor
make %{?_smp_mflags}

%install
rm -rf %{buildroot}
make pure_install DESTDIR=%{buildroot}
find %{buildroot} -type f -name .packlist -delete
%{_fixperms} -c %{buildroot}

%check
# Use our local Test::More if necessary
[ -d inc/Test/ ] && export PERL5LIB=$(pwd)/inc

make test

%clean
rm -rf %{buildroot}

%files
%if 0%{?_licensedir:1}
%license LICENSE
%else
%doc LICENSE
%endif
%doc Changes CONTRIBUTING.md README.md TODO.md
%{perl_vendorlib}/Specio.pm
%{perl_vendorlib}/Specio/
%{_mandir}/man3/Specio.3*
%{_mandir}/man3/Specio::Coercion.3*
%{_mandir}/man3/Specio::Constraint::AnyCan.3*
%{_mandir}/man3/Specio::Constraint::AnyDoes.3*
%{_mandir}/man3/Specio::Constraint::AnyIsa.3*
%{_mandir}/man3/Specio::Constraint::Enum.3*
%{_mandir}/man3/Specio::Constraint::Intersection.3*
%{_mandir}/man3/Specio::Constraint::ObjectCan.3*
%{_mandir}/man3/Specio::Constraint::ObjectDoes.3*
%{_mandir}/man3/Specio::Constraint::ObjectIsa.3*
%{_mandir}/man3/Specio::Constraint::Parameterizable.3*
%{_mandir}/man3/Specio::Constraint::Parameterized.3*
%{_mandir}/man3/Specio::Constraint::Role::CanType.3*
%{_mandir}/man3/Specio::Constraint::Role::DoesType.3*
%{_mandir}/man3/Specio::Constraint::Role::Interface.3*
%{_mandir}/man3/Specio::Constraint::Role::IsaType.3*
%{_mandir}/man3/Specio::Constraint::Simple.3*
%{_mandir}/man3/Specio::Constraint::Structurable.3*
%{_mandir}/man3/Specio::Constraint::Structured.3*
%{_mandir}/man3/Specio::Constraint::Union.3*
%{_mandir}/man3/Specio::Declare.3*
%{_mandir}/man3/Specio::DeclaredAt.3*
%{_mandir}/man3/Specio::Exception.3*
%{_mandir}/man3/Specio::Exporter.3*
%{_mandir}/man3/Specio::Helpers.3*
%{_mandir}/man3/Specio::Library::Builtins.3*
%{_mandir}/man3/Specio::Library::Numeric.3*
%{_mandir}/man3/Specio::Library::Perl.3*
%{_mandir}/man3/Specio::Library::String.3*
%{_mandir}/man3/Specio::Library::Structured.3*
%{_mandir}/man3/Specio::Library::Structured::Dict.3*
%{_mandir}/man3/Specio::Library::Structured::Map.3*
%{_mandir}/man3/Specio::Library::Structured::Tuple.3*
%{_mandir}/man3/Specio::OO.3*
%{_mandir}/man3/Specio::PartialDump.3*
%{_mandir}/man3/Specio::Registry.3*
%{_mandir}/man3/Specio::Role::Inlinable.3*
%{_mandir}/man3/Specio::Subs.3*
%{_mandir}/man3/Specio::TypeChecks.3*

%if ! %{old_test_more}
%files -n perl-Test-Specio
%{perl_vendorlib}/Test/
%{_mandir}/man3/Test::Specio.3*
%else
%exclude %{perl_vendorlib}/Test/
%exclude %{_mandir}/man3/Test::Specio.3*
%endif

%changelog
* Mon Feb 20 2017 Paul Howarth <paul@city-fan.org> - 0.36-1
- Update to 0.36
  - Inlined coercions would attempt to coerce for every type that matched the
    value given, instead of stopping after the first type (GH#11)
  - Inlined coercions did not include the inline environment variables needed
    by the type from which the coercion was being performed (GH#8)
  - When you use the same type repeatedly as coderef (for example, as a
    constraint with Moo), it will only generate its subified form once, rather
    than regenerating it each time it is de-referenced
  - Added an API to Specio::Subs to allow you to combine type libraries and
    helper subs in one package for exporting; see the Specio::Exporter docs for
    more details
- Update patch for building with old List::Util versions

* Mon Feb 13 2017 Paul Howarth <paul@city-fan.org> - 0.35-1
- Update to 0.35
  - Added Specio::Subs, a module that allows you to turn one or more library's
    types into subroutines like is_Int() and to_Int()
  - Added an inline_coercion method to Specio constraints
- Update patch for building with old List::Util versions

* Sat Feb 11 2017 Paul Howarth <paul@city-fan.org> - 0.34-2
- Rebuilt for https://fedoraproject.org/wiki/Fedora_26_Mass_Rebuild

* Mon Jan 30 2017 Paul Howarth <paul@city-fan.org> - 0.34-1
- Update to 0.34
  - Packages with Specio::Exporter can now specify additional arbitrary subs to
    exporter; see the Specio::Exporter docs for details
  - Importing the same library twice in a given package would throw an
    exception; the second attempt to import is now ignored
- Update patch for building with old List::Util versions

* Wed Jan 25 2017 Paul Howarth <paul@city-fan.org> - 0.33-1
- Update to 0.33
  - Fixed a mistake in the SYNOPSIS for Specio::Declare; the example for the
    *_isa_type helpers was not correct
  - Removed the alpha warning from the docs; this is being used by enough of my
    modules on CPAN that I don't plan on doing any big breaking changes without
    a deprecation first
- Update patch for building with old List::Util versions

* Fri Jan 13 2017 Paul Howarth <paul@city-fan.org> - 0.32-1
- Update to 0.32
  - Fixed a bug in the inlining for types create by any_can_type() and
    object_can_type(); this inlining mostly worked by accident because of some
    List::Util XS magic, but this broke under the debugger (GH#6,
    https://github.com/houseabsolute/DateTime.pm/issues/49)
- Update patch for building with old List::Util versions

* Mon Nov  7 2016 Paul Howarth <paul@city-fan.org> - 0.31-1
- Update to 0.31
  - The stack trace contained by Specio::Exception objects no longer includes
    stack frames for the Specio::Exception package
  - Made the inline_environment() and description() methods public on type and
    coercion objects
- Update patch for building with old List::Util versions

* Thu Oct 27 2016 Paul Howarth <paul@city-fan.org> - 0.30-2
- Break some build dependency cycles by avoiding Moo/Mouse/Moose when
  bootstrapping

* Sun Oct 16 2016 Paul Howarth <paul@city-fan.org> - 0.30-1
- Update to 0.30
  - Fix a bug with the Sub::Quoted sub returned by $type->coercion_sub; if a
    type had more than one coercion, the generated sub could end up coercing
    the value to undef some of the time and, depending on hash key ordering,
    this could end up being a heisenbug that only occurred some of the time
- Update patch for building with old List::Util versions

* Mon Oct 10 2016 Paul Howarth <paul@city-fan.org> - 0.29-1
- Update to 0.29
  - Document Specio::PartialDump because you may want to use it as part of the
    failure message generation code for a type
- Update patch for building with old List::Util versions

* Mon Oct  3 2016 Paul Howarth <paul@city-fan.org> - 0.28-1
- Update to 0.28
  - Added a Test::Specio module to provide helpers for testing Specio libraries
  - Fixed another bug with a subtype of special types and inlining
- Introduce sub-package perl-Test-Specio to avoid dependencies on Test::Fatal
  and Test::More in main package
- Update patch for building with old List::Util versions

* Sun Oct  2 2016 Paul Howarth <paul@city-fan.org> - 0.27-1
- Update to 0.27
  - Cloning a type with coercions defined on it would cause an exception
  - Creating a subtype of a special type created by *_isa_type, *_can_type, or
    *_does_type, or enum would die when trying to inline type constraint
  - Removed the never-documented Any type
  - Added documentation for each type in Specio::Library::Builtins
- Update patch for building with old List::Util versions

* Mon Sep 26 2016 Paul Howarth <paul@city-fan.org> - 0.26-1
- Update to 0.26
  - Require Role::Tiny 1.003003, which should fix some test failures
- Update patch for building with old List::Util versions

* Mon Sep  5 2016 Paul Howarth <paul@city-fan.org> - 0.25-1
- Update to 0.25
  - Calling {any,object}_{isa,does}_type repeatedly in a package with the same
    class or role name would die; these subs are now special-cased to simply
    return an existing type for the given name when they receive a single
    argument (the name of the class or role)
- Update patch for building with old List::Util versions

* Fri Jul  1 2016 Paul Howarth <paul@city-fan.org> - 0.24-1
- Initial RPM version
